#include <stdio.h>
#include <proto.h>

#include "gtnumber.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtNumber class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init number.
*
\************************************************************************/
GtNumber *GtMkNumber(char *n,GtManager *p,GtTag *tgs)
{
	GtNumber *obj;
	
	if(!(obj=new GtNumber()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtNumber object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init number.
*
\************************************************************************/
GtNumber *GtMkNumber(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtNumber *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkNumber(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty number.
*
\************************************************************************/
GtNumber::GtNumber(void)
{
	fmt="%d";
	num=0;
	sprintf(buf,fmt,num);
	len=strlen(buf);
}

/************************************************************************\
*
* Setup number.
*
\************************************************************************/
sbool GtNumber::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	XCharStruct xcs;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Number:
				num=t->d.val;
				break;
			case GT_Format:
				if(t->d.ptr)
					fmt=(char *)t->d.ptr;
				break;
		}
	}
	sprintf(buf,fmt,num);
	len=strlen(buf);
	gtfnt->StrExtents(buf,len,&xcs);
	fw=xcs.width;
	foff=xcs.ascent;
	fh=foff+xcs.descent;
	this->SetMinMaxSize(GTMAX(fw,1),GTMAX(fh,1),GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set number attributes.
*
\************************************************************************/
sbool GtNumber::Set(GtTag *tgs)
{
	GtTag *t;
	XCharStruct xcs;
	sbool ch=FALSE,fch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Number:
				num=t->d.val;
				ch=TRUE;
				break;
			case GT_Format:
				if(t->d.ptr)
					fmt=(char *)t->d.ptr;
				ch=TRUE;
				break;
			case GT_Font:
			case GT_FontList:
				fch=TRUE;
				break;
		}
	}
	if(ch)
	{
		sprintf(buf,fmt,num);
		len=strlen(buf);
	}
	if(fch || ch)
	{
		gtfnt->StrExtents(buf,len,&xcs);
		fw=xcs.width;
		foff=xcs.ascent;
		fh=foff+xcs.descent;
		this->SetMinMaxSize(GTMAX(fw,1),GTMAX(fh,1),GTS_MAX,GTS_MAX);
		this->Refresh();
	}
	return(TRUE);
}

/************************************************************************\
*
* Get number attributes.
*
\************************************************************************/
void GtNumber::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Number:
				*ptr=num;
				break;
			case GT_Format:
				*ptr=(long)fmt;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtNumber::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtNumber")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set number event mask.
*
\************************************************************************/
void GtNumber::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw number.
*
\************************************************************************/
sbool GtNumber::Draw(XEvent *xe)
{
	int xp,yp;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(flags & GTWF_NOWIN)
	{
		xp=xpos;
		yp=ypos;
	}
	else
	{
		xp=0;
		yp=0;
	}
	gtfore->DrawString(gtwin,xp+(xsize-fw)/2,yp+foff+(ysize-fh)/2,buf,len,FALSE);
	return(TRUE);
}
